//--------------------------------------------------------------------------//
// iq . 2006 . fast-code for the 4verts demo by RGBA                        //
//--------------------------------------------------------------------------//

#define WIN32_LEAN_AND_MEAN
#define WIN32_EXTRA_LEAN
#include <windows.h>
#include <GL/gl.h>

#include "intro.h"
#include "minigl/ext.h"
#include "ffx/ray_classic/ffx_rayclassic.inl"
#include "gfx/generate.h"
#include "gfx/funcs.h"

//---------------------------------------------------------------------

#define TEX0_ID 1

#define TEX0_XRES 512
#define TEX0_YRES 512

//---------------------------------------------------------------------

unsigned char mMemory[TEX0_XRES*TEX0_YRES*4];
int pid;

//---------------------------------------------------------------------
int intro_init( void )
{
    if( !EXT_Init() )
        return( 0 );

    unsigned char *bdata = mMemory;
    for( unsigned int v=0; v<TEX0_YRES; v++ )
    for( unsigned int u=0; u<TEX0_XRES; u++ )
    {
        generate1( bdata, (float)u*(1.0f/(float)(TEX0_XRES-1)), (float)v*(1.0f/(float)(TEX0_YRES-1)) );
        bdata += 4;
    }

    glBindTexture( GL_TEXTURE_2D, TEX0_ID );
    glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA8, TEX0_XRES, TEX0_YRES, 0, GL_BGRA, GL_UNSIGNED_BYTE, mMemory );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );

    //----------------------------------

    pid = oglCreateProgramObjectARB();
    int vsId = oglCreateShaderObjectARB( GL_VERTEX_SHADER_ARB );
    int fsId = oglCreateShaderObjectARB( GL_FRAGMENT_SHADER_ARB );
    //--------
    int vlen = sizeof(ray2_vertesxhader);
    int flen = sizeof(ray2_fragmenthader);
    const char *vs = ray2_vertesxhader;
    const char *fs = ray2_fragmenthader;
    oglShaderSourceARB( vsId, 1, (const char **)&vs, &vlen );
    oglShaderSourceARB( fsId, 1, (const char **)&fs, &flen );
    //--------
    oglCompileShaderARB( vsId );
    oglCompileShaderARB( fsId );
    oglAttachObjectARB( pid, fsId );
    oglAttachObjectARB( pid, vsId );
    //oglDeleteObjectARB( vsId );
    //oglDeleteObjectARB( fsId );
    oglLinkProgramARB( pid );
    //----------------------------------

    #ifdef DEBUG
    int     result;
    char    info[2048];
    oglGetObjectParameterivARB( vsId, GL_OBJECT_COMPILE_STATUS_ARB, &result );
    oglGetInfoLogARB( vsId, 1024, NULL, (char *)info );
    if( !result )
         return( 0 );
    oglGetObjectParameterivARB( fsId, GL_OBJECT_COMPILE_STATUS_ARB, &result );
    oglGetInfoLogARB( fsId, 1024, NULL, (char *)info );
    if( !result ) 
        return( 0 );
    oglGetObjectParameterivARB( pid, GL_OBJECT_LINK_STATUS_ARB, &result );
    oglGetInfoLogARB( pid, 1024, NULL, (char*)info );
    if( !result ) 
        return( 0 );
    #endif

    return( 1 );
}

//----------------------------------------

static float fparams[4*16] = 
{
  // objects
  0.0f,-5.0f, 0.0f,  0.60f,
  0.0f, 0.0f, 0.0f,  0.00f,
 -1.2f, 0.0f, 0.0f,  0.00f,
  0.0f, 0.0f, 0.0f,  0.00f,

  0.0f, 1.0f, 0.0f,  2.00f,
  0.0f,-1.0f, 0.0f,  2.00f,
  // colors
  1.0f, 0.75f, 0.75f, 1.0f,
  1.0f, 0.75f, 0.90f, 1.0f,
  1.0f, 0.90f, 0.75f, 3.0f,
  1.0f, 0.90f, 0.90f, 3.0f,
  1.0f, 1.00f, 1.00f, 2.0f,
  1.0f, 1.00f, 1.00f, 2.0f,
};

void intro_do( long time )
{
    conf_rayclassic( fparams, time );

    oglUseProgramObjectARB( pid );
    oglUniform4fvARB( 0, 6+6+4, fparams );
    oglUniform1iARB( 16, 0 );

    #ifdef DEBUG
    if( oglGetUniformLocationARB( pid, "fpar00" )!=0 ) DebugBreak();
    if( oglGetUniformLocationARB( pid, "tex00" )!=16 ) DebugBreak();
    #endif

    glEnable( GL_TEXTURE_2D );
    glBindTexture( GL_TEXTURE_2D, TEX0_ID );
    glRects( -1, -1, 1, 1 );
}

