#define XRES        1920
#define YRES        1080

//#define ERRORMSG              // activa printado mensajes de error
#define CLEANDESTROY          // destruir ventana, glContext, ...

#define WIN32_LEAN_AND_MEAN
#define WIN32_EXTRA_LEAN
#include <windows.h>
#include <mmsystem.h>
#include "../../intro.h"
#include "../../mzk.h"

//----------------------------------------------------------------------------

HDC     hDC;
HGLRC   hRC;
HWND    hWnd;
short   myMuzik[MZK_NUMSAMPLES+22];

//----------------------------------------------------------------------------
#ifdef __cplusplus
extern "C" 
{
#endif
int  _fltused = 0;
#ifdef __cplusplus
}
#endif

static PIXELFORMATDESCRIPTOR pfd =
    {
    sizeof(PIXELFORMATDESCRIPTOR),
    1,
    PFD_DRAW_TO_WINDOW|PFD_SUPPORT_OPENGL|PFD_DOUBLEBUFFER,
    PFD_TYPE_RGBA,
    32,
    0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0,
    32, 0, 0,
    PFD_MAIN_PLANE,
    0, 0, 0, 0
    };

static int wavHeader[11] = {
    0x46464952, 
    MZK_NUMSAMPLES*2+36, 
    0x45564157, 
    0x20746D66, 
    16, 
    WAVE_FORMAT_PCM|(MZK_NUMCHANNELS<<16), 
    MZK_RATE, 
    MZK_RATE*MZK_NUMCHANNELS*sizeof(short), 
    (MZK_NUMCHANNELS*sizeof(short))|((8*sizeof(short))<<16),
    0x61746164, 
    MZK_NUMSAMPLES*sizeof(short)
    };

//----------------------------------------------------------------------------

__forceinline int window_init( void )
{
    unsigned int    PixelFormat;
    DEVMODE         dmScreenSettings;

    dmScreenSettings.dmSize       = sizeof(DEVMODE);
    dmScreenSettings.dmFields     = DM_BITSPERPEL|DM_PELSWIDTH|DM_PELSHEIGHT;
    dmScreenSettings.dmBitsPerPel = 32;
    dmScreenSettings.dmPelsWidth  = XRES;
    dmScreenSettings.dmPelsHeight = YRES;

    if( ChangeDisplaySettings(&dmScreenSettings,CDS_FULLSCREEN)!=DISP_CHANGE_SUCCESSFUL)
        return( 0 );

    //while( ShowCursor( 0 )>=0 );  // hide cursor
    ShowCursor(0);  // hide cursor

    hWnd = CreateWindowEx( WS_EX_APPWINDOW, "STATIC", ".", WS_VISIBLE|SS_OWNERDRAW|WS_POPUP,
                           0, 0, XRES, YRES, 0, 0, (HINSTANCE)0x00400000, 0 );
    
    if( !hWnd ) return( 0 );

    hDC = GetDC(hWnd);
    //if( !hDC ) return( 0 );

    PixelFormat = ChoosePixelFormat(hDC,&pfd);
    //if( !PixelFormat ) return( 0 );

    if( !SetPixelFormat(hDC,PixelFormat,&pfd) )
        return( 0 );

    hRC = wglCreateContext(hDC);
    //if( !hRC ) return( 0 );

    if( !wglMakeCurrent(hDC,hRC) )
        return( 0 );
    
    return( 1 );
}

__forceinline void window_end( void )
{
    #ifdef CLEANDESTROY
    if( hRC )
    {
        wglMakeCurrent( 0, 0 );
        wglDeleteContext( hRC );
    }

    if( hDC  ) ReleaseDC( hWnd, hDC );
    if( hWnd ) DestroyWindow( hWnd );
    #endif

    ChangeDisplaySettings( 0, 0 );
    //while( ShowCursor( 1 )<0 ); // show cursor
    ShowCursor(1);
}


//===============================================

void entrypoint( void )
{
    int         done=0;
    long        to=0;

    // create window
    if( !window_init() )
        goto endend;

    // init intro
    if( !intro_init() )
        goto endend;

    // calculate music
    mzk_init( myMuzik+22 );

    // and play it 
    memcpy( myMuzik, wavHeader, 44 );
    sndPlaySound( (const char*)&myMuzik, SND_ASYNC|SND_MEMORY );

    // demo loop
    while( !done )
    {
        MSG msg;
        //if( PeekMessage(&msg,0,0,0,PM_REMOVE) )
        PeekMessage(&msg,0,0,0,PM_REMOVE);
        {
            if( msg.message==WM_KEYDOWN    && msg.wParam==VK_ESCAPE ) done = 1;   // ESC
            if( msg.message==WM_SYSCOMMAND && msg.wParam==SC_CLOSE  ) done = 1;   // ALT F4
        }

        long t = timeGetTime(); if( !to ) to=t; t=t-to;
        //long t = msg.time; if( !to ) to=t; t=t-to;

        intro_do( t );

        if( t>(MZK_DURATION*1000) ) done = 1;

        SwapBuffers( hDC );
        //wglSwapLayerBuffers( hDC, WGL_SWAP_MAIN_PLANE );
    }

    sndPlaySound(0,0);

endend:
    window_end();

    #ifdef ERRORMSG
    if( !done ) MessageBox( 0, "no OpenGL 2.0?!\nno GLSL 1.0?", "error", MB_OK|MB_ICONEXCLAMATION );
    #endif

    ExitProcess(0);
}
